/*
   A component which displays a text
   Copyright (C) 1997  Martin Vogt

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation.                                       

   For more information look at the file COPYRIGHT in this package

*/



import java.awt.*;
import java.awt.event.*;


/**
 * A component that displays
 * a Text.
 * <p>
 * The Component can be marked.
 *
 */



public class TextMarkComponent extends Canvas
                                 implements ItemSelectable,
                                            MouseListener {

  transient ItemListener itemListener;

    /**
     * The label.
     */
    String label;


  Color markBackgroundColor=Color.blue;
  Color markForegroundColor;

  Color backgroundColorBuffer;
  Color foregroundColorBuffer;

  boolean mark=false;

  /**
   * These Insets extends the TextComponent to the top,bottom,left,right
   *
   * normally the markded area doesnt look "nice",if it exactly
   * surronds the Text.
   * The Insets default to (2,2,2,2)
   */
  Insets insets;

  /**
   * Constructs an empty label.
   */
    public TextMarkComponent() {
	this("");
    }

    /**
     * Constructs a new label with the specified String of text.
     * @param label the text that makes up the label
     */
    public TextMarkComponent (String label) {
      markBackgroundColor=markBackgroundColor.darker();
      insets=new Insets(2,2,2,2);
      setText(label);
      addMouseListener(this);
    }

  public Dimension getPreferredSize() {
    FontMetrics fm;
    Dimension back=new Dimension(0,0);

    fm=Toolkit.getDefaultToolkit().getFontMetrics(getFont());
    back=new Dimension(fm.stringWidth(label),fm.getHeight());

    back.height+=insets.top+insets.bottom;
    back.width+=insets.left+insets.right;
    return back;
  }


  public Dimension getMinimumSize() {
    return getPreferredSize();
  }




  public void paint(Graphics g) {
    Dimension size=getSize();
    FontMetrics fm;

    if (mark) {
      setBackground(markBackgroundColor);
      setForeground(markForegroundColor);
    }
    fm=Toolkit.getDefaultToolkit().getFontMetrics(getFont());
    g.drawString(label,insets.left,
		 size.height-insets.bottom-fm.getDescent());


  }





    /**
     * Gets the text of this label.
     * @see #setText
     */
    public String getText() {
	return label;
    }

    /**
     * Sets the text for this label to the specified text.
     * @param label the text that makes up the label
     * @see #getText
     */
    public void setText(String label) {
	if (label != this.label && (this.label == null
				    || !this.label.equals(label))) {
	    this.label = label;
	    invalidate();
	}
    }



   /**
   * Set the mark for the text
   *
   */
  public void setMark(boolean mark) {
    if (this.mark != mark) {  // wechsel
      this.mark=mark;
      if (mark == true) {
	backgroundColorBuffer=getBackground();
	foregroundColorBuffer=getForeground();
	if (markForegroundColor == null) {
	  markForegroundColor=backgroundColorBuffer;
	}
	setForeground(markForegroundColor);
	setBackground(markBackgroundColor);
      } else {
	setForeground(foregroundColorBuffer);
	setBackground(backgroundColorBuffer);
      }
    }
    invalidate();
  }


  /**
   * Gets the mark Variable for the text
   *
   */
  public boolean getMark() {
    return(mark);
  }



  public Color getMarkBackgroundColor() {
    return markBackgroundColor;
  }

  public void setMarkBackgroundColor(Color newMarkColor) {
    markBackgroundColor = newMarkColor;
    invalidate();
  }

  public Color getMarkForegroundColor() {
    return markForegroundColor;
  }

  public void setMarkForegroundColor(Color newMarkColor) {
    markForegroundColor = newMarkColor;
    invalidate();
  }




  // Interface:  ItemSelectable [Start]

    /**
     * Adds the specified item listener to recieve item events from
     * this TextMarkComponent
     * @param l the item listener
     */
    public void addItemListener(ItemListener l) {
        itemListener = AWTEventMulticaster.add(itemListener, l);
    }

    /**
     * Removes the specified item listener so that it no longer
     * receives item events from this TextMarkComponent
     * @param l the item listener
     */
    public void removeItemListener(ItemListener l) {
        itemListener = AWTEventMulticaster.remove(itemListener, l);
    }


   /**
     * Returns the an array (length 1) containing the TextMarkComponent
     * label or null if the Component is not selected.
     * @see ItemSelectable
     */
    public Object[] getSelectedObjects() {
        if (mark) {
            Object[] items = new Object[1];
            items[0] = label;
            return items;
        }
        return null;
    }

    // Interface:  ItemSelectable [End]


    // Interface : MouseListener [Start]

    /**
     * The mouse click on a TextMarkComponent ist mapped
     * to an ItemEvent
     */
    public void mouseClicked(MouseEvent e) {
      ItemEvent ev;
      //System.out.println("Mouse clicked");
      if (mark == true) {
        ev=new ItemEvent(this,ItemEvent.ITEM_STATE_CHANGED,"",
                            ItemEvent.DESELECTED);
      } else {
        ev=new ItemEvent(this,ItemEvent.ITEM_STATE_CHANGED,"",
                            ItemEvent.SELECTED);
      }
      processEvent(ev);
    }

    /**
     * no functionality behind
     */
    public void mousePressed(MouseEvent e) {
      //System.out.println("Mouse pressed");
    }

    /**
     * no functionality behind
     */
    public void mouseReleased(MouseEvent e) {
      //System.out.println("Mouse released");
    }


    /**
     * no functionality behind
     */
    public void mouseEntered(MouseEvent e) {
      //System.out.println("Mouse entered");
    }

    /**
     * no functionality behind
     */
    public void mouseExited(MouseEvent e) {
      //System.out.println("Mouse exited");
    }

    // Interface : MouseListenr [End]





    /**
     * Processes events on this checkbox. If the event is an ItemEvent,
     * it invokes the handleItemEvent method, else it calls its
     * superclass's processEvent.
     * @param e the event
     */
    protected void processEvent(AWTEvent e) {
        if (e instanceof ItemEvent) {
            processItemEvent((ItemEvent)e);
            return;
        }
        super.processEvent(e);
    }

    /**
     * Processes item events occurring on this checkbox by
     * dispatching them to any registered ItemListener objects.
     * NOTE: This method will not be called unless item events
     * are enabled for this component; this happens when one of the
     * following occurs:
     * a) An ItemListener object is registered via addItemListener()
     * b) Item events are enabled via enableEvents()
     * @see Component#enableEvents
     * @param e the item event
     */
    protected void processItemEvent(ItemEvent e) {
        if (itemListener != null) {
            itemListener.itemStateChanged(e);
        }
    }









    /**
     * Returns the parameter String of this label.
     */
    protected String paramString() {

	return super.paramString() + ",label=" + label+",mark="+mark;
    }
}

