

/*
   AWTNode is a tree-view implementation in Java.
   Copyright (c) 1997 Fraunhofer-Einrichtung fuer Experimentelles
                      Software Engineering (IESE)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation.

   For more information look at the file COPYRIGHT in this package

*/




package de.unikl.awtnode;


import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.lang.*;
import de.unikl.awtnode.RescanNode;



/**
  The Class AWTNode implements a new Java Component.
  <p>
  With this class you are able to build an tree-structure which
  can be shown on the screen. (e.g. a File/Directory tree like
  in the Windows explorer)


  <em>What I have to do: -how to create a tree-</em><p>

  First you create a "root"-Node and then you insert into this
  node a new AWT components. On the one hand these components
  can be "ordinary" components like Buttons or Lables
  on the other hand these components can be new AWTNodes.
  If you insert a new AWTNode to your "root"
  node with the insert(...) method you just have merged two trees
  together, and you have build a tree with excatly two nodes.
  If you continue to insert other instances of AWTNode the
  tree gets "bigger".

  <p>
  as a result you can say:
  The tree is build out of single AWTNodes

  <p>
  example:
  <p><pre><code>

  root =new AWTNode(new Button("root-node"));        // the whole tree

  root.insert(new Button("autoexec.bat"));             // a simple "File"
  root.insert(new AWTNode(new Button("[DOS]")));       // a Subdir
  </code>
  <p>
  <em>Constructs:</em><p>

     [c]-[root]
	       |
	       +------autoexec.bat
	       |
	       +------[c]-[DOS]
  </pre>

  and [c] means a Checkbox.
  <p>In front of each AWTtree we draw a Checkbox.
  <p><p>

  The interface to build a tree is quite simple. (You shouldnt need
  the superclasses like RescanNode or NodeContainer,
  the best thing is to ignore them)

  <p>
  In general a AWT"tree" will work as expected and you wont need
  the superclasses.


  <em>internal implementation:</em><p>

  The source code is a typical "treeimplemetation"
  Like in the "ordinary" tree-structure there is a "key" and the
  "pointer" to the sons

  <pre>
                +------+
                | key  |
     +----------+------+---+
     |      |              |....
     son 1  son 2          son i
  </pre>
  The only thing which might be confusing is the fact that
  the "key" in this implementation is a AWT-component (and not
  an integer) But this is because this implemennts an *AWT* tree,
  and thus the "keys" have to be components.

  <p>
  In general the tree is filled with the insert methods. Because
  this is a recursive approch most of the methode in AWTNode
  work recusivley on the "sons"
  <p>
  You can connect a ActionListener with a AWTNode node.
  Then this ActionListener receives "open" and "close" events.
  */





public class AWTNode extends RescanNode {


    private static final String base = "awtnode";
    private static int nameCounter = 0;

    private static final String OPEN = "open";
    private static final String CLOSE = "close";


    transient ActionListener actionListener;


  Component key;


  public AWTNode() {
    this(new Canvas());

  }


  /**
   * Constructs a new AWTtree with a component as its graphical representation
   */

 public AWTNode(Component key) {
   super(key);
   this.key=key;
   setName(base + nameCounter++);
  }


  /**
   * returns the key-Component of this node
   */

  public Component getKey() {
    return key;
  }


  /**
   * sets a new Key for this AWTNode
   */
  public void setKey(Component key) {
    setDescription(key);
  }


  /**
   * returns the number ob sons in the AWTNode
   */

  /* Note:
   * here we use the internal implemtentation. All sons in the
   * NodeContainer have the Identifier "stdNode" there ar other
   * identifiers as well, but they are only used during layout.
   * We have to ignore them if we only want the sons!
   */
  public int countSons() {
    return getComponentCount("stdNode");
  }



  /***
   * returns all the sons of this AWTNode
   * sons are simple Components as well as AWTNodes...
   *
   * @return array of Sons
   */
  public Component[] getSons() {

    return getComponents("stdNode");
  }



  /**
   * Inserts a son to this Node
   * A son may be an ordinary Component or an AWTNode.
   */
  public void insert(Component son) {
    super.add(son);
  }



  /**
   * removes all sons from this node
   */
  public void removeAllSons() {
    removeAll("stdNode");
  }

    /**
     * "opens" or "closes" a AWTNode
     *
     * The "open" or "close" event is passed
     * to the listner before the
     * event is passed to its superclass
     * @param check  false="open" the AWTNode /true ="close" it
     */

    public void setState(boolean check) {
        ActionEvent actionevent;
        if (check == true) {
            actionevent=new ActionEvent(this,
                        ActionEvent.ACTION_PERFORMED,
                        AWTNode.CLOSE);
        } else {
            actionevent=new ActionEvent(this,
                        ActionEvent.ACTION_PERFORMED,
                        AWTNode.OPEN);
        }
        processEvent(actionevent);
        super.setState(check);
    }


    //*********************************************************
    //*
    //* implement addActionListener...[Start]
    //*
    //*********************************************************


   /**
     * Adds the specified action listener to receive action events
     * from this button.
     * @param l the action listener
     */
    public void addActionListener(ActionListener l) {
        actionListener = AWTEventMulticaster.add(actionListener, l);
    }

    /**
     * Removes the specified action listener so it no longer receives
     * action events from this button.
     * @param l the action listener
     */
    public void removeActionListener(ActionListener l) {
        actionListener = AWTEventMulticaster.remove(actionListener, l);
    }


    /**
     * Processes events on this button. If the event is an ActionEvent,
     * it invokes the processActionEvent method, else it invokes its
     * superclass's processEvent.
     * @param e the event
     */
    protected void processEvent(AWTEvent e) {
        if (e instanceof ActionEvent) {
            processActionEvent((ActionEvent)e);
            return;
        }
        super.processEvent(e);
    }

     /**
     * Processes action events occurring on this button by
     * dispatching them to any registered ActionListener objects.
     * NOTE: This method will not be called unless action events
     * are enabled for this component; this happens when one of the
     * following occurs:
     * a) An ActionListener object is registered via addActionListener()
     * b) Action events are enabled via enableEvents()
     * @see Component#enableEvents
     * @param e the action event
     */
    protected void processActionEvent(ActionEvent e) {
        if (actionListener != null) {
            actionListener.actionPerformed(e);
        }
    }
    //*********************************************************
    //*
    //* implement addActionListener...[END]
    //*
    //*********************************************************






}







