/*
   ConnectionLinePoints draws lines
   Copyright (c) 1997 Fraunhofer-Einrichtung fuer Experimentelles
                      Software Engineering (IESE)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation.

   For more information look at the file COPYRIGHT in this package

*/


package de.unikl.awtnode;




import de.unikl.awtnode.LinePoints;
import java.util.*;
import java.awt.*;

/**
  The class ConnectionLinePoints is a lightweight component.
  The basic idea is, that this class stores only lines, given
  with a StartPoint (x1,y1) and an endpoint (x2,y2)
  <p>
  Important: This class uses the graphics-context of its container
  <p>
  If you call the paint method of this class, all the lines
  are painted in the parents graphic context. Thus this
  lightweigth component have to know ist parent.
  You can set the parent of the Component with the method:
  <pre><code>
  setParent(Component parent)
  </pre></code>

  <p>
  To get a higher performance between the steps:

  <pre><p>
    1. hide the actual Objekt
    2. fill the Objekt with new lines
    3. paint these lines
  </pre><p>
  there is a possibility to put the "invalid" lines in a
  Buffer. If there are invalid lines in the buffer the
  paint method of this component first hide these
  invalid lines, and then it paints the new lines.


  <p>
  <H4>For what is this class good for?</H4>
  <p>
  As an example you can store all the lines in it, which you need
  if you implement a tree-view. In a tree-view you have lines
  from each parent-Node to its son-nodes.

  <p>
  <H4>internal implementation:</H4>
  <p>
   The lines are stored in a vector structure.

  */



public class ConnectionLinePoints extends Component{



  /**
   * In this Vector we store the LinePoint-Object.
   * These are Objects wich consists of two Point:
   * the StartPoint of the Line, and the EndPoint of the Line
   * This Vector stores the current lines. The current lines
   * are these lines which are added after the last call to
   * the method restLines()
   */
  Vector currentLines;



  /**
   * In this Vector we store all of these LinePoints
   * which are invalid, but currently stays on the screen.
   * In each paint method we draw these lines in the background
   * color and afterwards clear the Vector
   */
   Vector oldLines;



  /**
   * we store whether this class is painted or not if we call the
   * paint method.
   * default is lpainted=true
   */
  /*
   * Note: sometimes you want to show the ConnectionLines
   * and sometimes not. You dont need to check whether to call
   * the paint method of this component or not, you simply
   * call the paint()-method of this object and it decides internally
   * whether it draws itsself or not*/
  boolean lpainted;


  /**
   * If you specify the parent with the setParent(..) methode
   * this object paints itsself IN THE PARENT GRAPHICS KONTEXT
   */
  Component parent;





  /**
   * Constructs an emty Object of ConnectionLines which sets visible=true
   */
  ConnectionLinePoints() {
    currentLines=new Vector();
    oldLines=new Vector();
    lpainted=true;
  }

  /**
   * adds a new Line to the Object
   * Note:
   * Lines are stored in the coordinate space of the Container,
   * or Component to which this Object belongs to.
   * This is important if you construct the graphics context
   * in which the lines should draw themselves
   *
   * @param a the Startpoint of the line (in parents coordiante space)
   * @param b the EndPoint of the line (in parents coordiante space)
   */
  public void addElement(Point a, Point b) {
    currentLines.addElement(new LinePoints(a,b));
  }

  /**
   * returns the number of currentLines-Elements in this Object
   */
  public int getLineCount() {
    return currentLines.size();
  }


  /**
   * return the LinePoints Object at the given index position
   */
  public LinePoints elementAt(int pos) {
    LinePoints back=null;

    try {
      back=(LinePoints)currentLines.elementAt(pos);
    } catch (ArrayIndexOutOfBoundsException e) {
      System.out.println("Exception ArrayIndexOutOfBoundsException ");
      System.out.println("in ConnectionLinePoints :ElementAt");
      System.out.println("exception: " + e.getMessage());
      e.printStackTrace();
      System.exit(-1);
    }
    return back;
  }

  /**
   * Clears the currentLine variable, and append the current lines
   * to the oldLines object.
   */

  public void resetLines() {
    int i;
    int n=currentLines.size();
    LinePoints tmp;
    // first we copy the lines to the oldLines Object
    for(i=0;i<n;i++) {
    try {
      tmp=(LinePoints)currentLines.elementAt(i);
      oldLines.addElement(tmp);
    } catch (ArrayIndexOutOfBoundsException e) {
      System.out.println("Exception ArrayIndexOutOfBoundsException ");
      System.out.println("in ConnectionLinePoints :resetLines");
      System.out.println("exception: " + e.getMessage());
      e.printStackTrace();
      System.exit(-1);
    }
    }
    // then we clear the current Lines
    currentLines.removeAllElements();
  }



  /**
   * returns whether the Object is painted in the paint method
   * or not.
   */
  public boolean isPainted() {
    return lpainted;
  }


  /**
   * This Method is important if the Object should draw its
   * Lines in the parents graphic context.
   * You must specify a parent or the Object cannot get
   * the graphic context and cannot be drawn.
   */

  public void setParent(Component parent) {
    this.parent=parent;
  }


  /**
   * This method sets the painted flag for the Object.
   */
  public void setPainted(boolean lpaint) {
    Graphics g;

    lpainted=lpaint;
    if (parent == null) return;
    g=parent.getGraphics();
    paint(g);

  }




  /**
   * Paints the object. If isPainted()==false, we hide the oldLines
   * and no currentLines are drawn.
   * otherwise we hide the oldLines as well, but draw the currentLines
   * in the parent Grapics Context.
   */
  public void paint(Graphics g) {
        int i;
        int n=getLineCount();
        Graphics gparent;

        if (parent == null) return;
        gparent=parent.getGraphics();
        hideOldLines();
        if (lpainted == false) return;
        // else : show current Lines

        for(i=0;i<n;i++) {
    	    LinePoints line=elementAt(i);
    	    gparent.drawLine(line.a.x,line.a.y,line.b.x,line.b.y);
        }

  }

    /**
     * Hides the old Lines. The oldLines are drawn in the
     * backgroundcolor of the parent
     */

    private void hideOldLines() {
        int i;
        int n=oldLines.size();
        Graphics gparent;
        Color oldColor;

        if (parent == null) return;
        gparent=parent.getGraphics();

        // hide oldLines
        oldColor=parent.getForeground();
        gparent.setColor(parent.getBackground());

        for(i=0;i<n;i++) {
        	LinePoints line;
        	try {
        	      line=(LinePoints)oldLines.elementAt(i);
        	      gparent.drawLine(line.a.x,line.a.y,line.b.x,line.b.y);
        	} catch(ArrayIndexOutOfBoundsException e) {
              System.out.println("ArrayIndexOutOfBoundsException ");
              System.out.println("in ConnectionLinePoints :hideOldLines");
              System.out.println("exception: " + e.getMessage());
              e.printStackTrace();
              System.exit(-1);
            }
        }
        gparent.setColor(oldColor);
        // oldLines hided
    }



    public String toString() {
        String back;
        back= getClass().getName() + "[";
        back=back+"lpainted="+lpainted;
        return back+"]";
    }
}

