/*
   FileNodeLayout lays ot Components in a File-tree
   Copyright (c) 1997 Fraunhofer-Einrichtung fuer Experimentelles
                      Software Engineering (IESE)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation.

   For more information look at the file COPYRIGHT in this package

*/


package de.unikl.awtnode;


import java.awt.*;
import de.unikl.awtnode.NodeContainer;
import de.unikl.awtnode.pmCheckbox;


/**
  This LayoutManager layouts the Components in a "FileTree" eg:

  <p><pre>

  ...+ ----hgapRoot
  +--|----------------------------------------+
  |  Y                                        |
  | * AAAAAAAAAA                              |
  |           *                               | vgap1 <-- vertikal spacing
  |           +*******BBBBBBBBBB              |
  | 	      *                               |
  |           -*******CCCCCCCCCC              |
  |           *           *                   |
  |           *           +******DDDDDDDDDDD  |
  |           *                               |
  |           +*******EEEEEEEEEE              |
  |                                           |
  +-------------------------------------------+
  .....|hgap1|  |hgap2|

  .....  ^       ^
  .....  +-------+----- horizontal spacing in this Layout Manager

  <p></pre>
  <H4> Note: </H4>

  Only works properly with "Node"-Containers !!!!!!!
  and if the size of <CheckComponent> in NodeContainer is
  small in contrast to the size of <description>
  (In fact it must be granted
  CheckComponent.height() <= Description.height)

  <p>


  <H4>Internal implementation:</H4>

  Like in BorderLayout in the JDK I have implemented identifiers which
  describe the "kind" of a Component in the NodeContainer. They don't
  call "South" "West" but :

  <p><pre>
  "stdNode"
  "description"
  "check"
  "thischeck"
  <p></pre>

  The identifier "stdNode" describes all Components which are "add"
  with the normal Mehtod : add(Component comp) to the NodeContainer.
  I have overloaded this Method in NodeContainer. It simply invokes
  add("stdNode",comp)
  Thus these are the "contents" of the Container.
  (In a FileNodeLayout these are the File-Names)

  <p>
  The identifiers "description" and "check" describe Components which
  are for the graphical representation of the NodeContainer itself.
  The "check" allows to hide/show the "stNode" Components and
  the "description" Component is "its description-Name".
  (In a FileNodeLayout the name of the directory)

  <p>
  The "thischeck" describes the Checkbox of the NodeContainer to which
  this Layout belongs.
  <H4>Note:</H4>
  the "thischeck" is only in the Conatiner if and only if the this
  NodeContainer has nor parent NodeContainer. Thus the "thischeck"
  is only a member of the NodeContainer if it is the "root" NodeContainer.
  Depending ob the Status getpaint() we draw this "root"-Checkbox or not.



  **/


public class FileNodeLayout implements LayoutManager {


    int hgap1;
    int hgap2;
    int hgapRoot;
    int vgap1;

    /**
     * Constructs a new FileNodeLayout with  standard-spacing

     */
    public FileNodeLayout() {
	this(8, 11, 4,10);
    }


    /**
     * Constructs a new FileNodeLayout with the specified gap values
     *
     * @param hgap1 the "left" horizontal gab variable
     * @param hgap2 the "right" horizontal gap variable
     * @param vgap1 the vertical gap variable
     */
    public FileNodeLayout(int hgap1, int hgap2, int vgap1,int hgapRoot) {
	this.hgap1 = hgap1;
	this.hgap2 = hgap2;
	this.vgap1 = vgap1;
	this.hgapRoot=hgapRoot;
    }

    /**
     * Adds the specified component to the layout. Not used by this class.
     * @param name the name of the component
     * @param comp the the component to be added
     */
    public void addLayoutComponent(String name, Component comp) {
    }

    /**
     * Removes the specified component from the layout. Not used by
     * this class.
     * @param comp the component to remove
     */
    public void removeLayoutComponent(Component comp) {
    }



    /**
     * Returns the preferred dimensions for this layout given the components
     * in the specified target container.
     * Note: The target must be a NodeContainer (!)
     * @param target the component which needs to be laid out
     * @see Container
     * @see #minimumLayoutSize
     */
    public Dimension preferredLayoutSize(Container target) {
        NodeContainer node;
	Dimension dim = new Dimension(0, 0);
	Dimension tmp1;
        Component description;
	pmCheckbox thischeck;
	int tmphgapRoot=0;
	int nmembers = target.getComponentCount();



	if (!(target instanceof NodeContainer)) {
	  System.out.println("Sorry only works with NodeContainers!");
	  System.out.println("call to  preferredLayoutSize");
	  System.exit(-1);
	}
	node=(NodeContainer)target;   // now the cast
	target=null;

    //System.out.println("node="+node.toString());
	description=node.getDescription();
	thischeck=node.getCheckComponent();

	tmp1=new Dimension(0,0);
	if (description.isVisible()) {
	  tmp1=description.getPreferredSize();
	}
        // This is a FileNodeLayout, thus we arrange the Components like:
        // <Description>
        // and the preferred Dimension calculates in this way:
        dim.width+=tmp1.width;
        dim.height+=tmp1.height;


	// Now a word about the tmphgapRoot Variable. In general it is not
	// clear wether we draw a "root" checkbox or not.
	// Each time we calculate the width we would have to check
	// for the "thischeck"-box as well.
	// First we determine if this is a root-NodeContainer,
	// Then we woul check wether the "root" Checkbox should be drawn
	// Then we calculate the new width (and height)

	// To speed up things we calculate one time the tmphgapRoot
	// variable. If we NOT draw the "root" checkbox the value is 0

	if (thischeck.getParent() == node) { // check belongs to this NodeCon?
	  if ((thischeck.getPaint() == true) && (thischeck.isVisible())){
	    Dimension checkdim=thischeck.getPreferredSize();
	    dim.width+=checkdim.width+hgapRoot;
	    dim.height=Math.max(checkdim.height,dim.height);
	    tmphgapRoot=checkdim.width+hgapRoot;
	  }
	}


        // This is a FileNodeLayout, we have to add
        // hgap1,hgap2

        dim.width=Math.max(dim.width,hgap1+hgap2+tmphgapRoot);


	for (int i = 0 ; i < nmembers ; i++) {
	    Component m = node.getComponent(i);
	    String s;
	    s=node.getIdentifier(m);
	    if (m.isVisible() && s.equals("stdNode")) {
		Dimension d = m.getPreferredSize();

        // This is a FileNodeLayout,
		// we have to add hgap1 and hgap2
		dim.width = Math.max(dim.width,
				     d.width+hgap1+hgap2+tmphgapRoot);
		dim.height += vgap1+d.height;

	    }
	}
	Insets insets = node.getInsets();
	dim.width += insets.left + insets.right ;
	dim.height += insets.top + insets.bottom ;
	return dim;
    }



    /**
     * Returns the minimum dimensions needed to layout the components
     * contained in the specified target container.
     * Note: The parameter is a NodeContainer
     * (and we call preferredLayoutSize...)
     * @param target the component which needs to be laid out
     * @see #preferredLayoutSize
     */
    public Dimension minimumLayoutSize(Container target) {

	return preferredLayoutSize(target);
    }



    /**
     * Lays out the container. This method will actually reshape the
     * components in the target in order to satisfy the constraints of
     * the FileNodeLayout object.
     * Note: The parameter must be a NodeContainer (!)
     * @param target the specified component being laid out.
     * @see Container
     */
    public void layoutContainer(Container target) {
	NodeContainer node;
	NodeContainer anode;
	Insets insets = target.getInsets();
	Component description;
	ConnectionLinePoints ConnPoints;
	pmCheckbox check;
	pmCheckbox thischeck;
	Dimension d=new Dimension(0,0);
	int nmembers = target.getComponentCount();
	int x = insets.left;
	int y = insets.top;
	Point StartPoint;
	Point EndPoint;


	if (!(target instanceof NodeContainer)) {
	  System.out.println("Sorry only works with NodeContainers!");
	  System.out.println("call to  layoutContainer");
	  System.exit(-1);
	}

	node=(NodeContainer)target;  // now the cast
	target=null;


    description=node.getDescription();
	ConnPoints=node.getConnectionLinePoints();
	ConnPoints.resetLines();  // new Layout, new connectionlines
	thischeck=node.getCheckComponent();


	if (thischeck.getParent() == node) { // check belongs to this NodeCon?
	  if ((thischeck.getPaint() == true) && (thischeck.isVisible())){
	    Dimension checkdim=thischeck.getPreferredSize();
	    Component tmpdesc=description;
	    Dimension ddesc;
	    Dimension dtmp=new Dimension(0,0);

	    thischeck.setSize(checkdim.width,checkdim.height);
	    // Now do a proper layout for the "root"-checkbox
	    if (tmpdesc instanceof NodeContainer) {
	      NodeContainer nodedesc=(NodeContainer)tmpdesc;
	      tmpdesc=nodedesc.getDescription();
	    }
	    ddesc=tmpdesc.getPreferredSize();
	    dtmp.height=(int)ddesc.height/2-(int)checkdim.height/2;
	    dtmp.width=checkdim.width/2;

	    thischeck.setLocation(dtmp.width,dtmp.height);
	    x=checkdim.width+hgapRoot;

	    ConnPoints.addElement(new Point(checkdim.width,ddesc.height/2),
				  new Point(x,ddesc.height/2));

	  }
	}




	// Now the Layout for description
	// We are a FileNodeLayout
	if (description.isVisible()) {
          d=description.getPreferredSize();
	  description.setSize(d.width, d.height);
	  description.setLocation(x,y);
        }

	x+=hgap1+hgap2;
	y+=d.height+vgap1;

	StartPoint=new Point(x-hgap2,y-vgap1);  // Point under "description"
	for (int i = 0 ; i < nmembers ; i++) {
	    Component m;
	    m=node.getComponent(i);
	    Dimension dcheck;
	    int xtmp;
	    int ytmp;
	    int j;

	    if (m.isVisible() && node.getIdentifier(m).equals("stdNode")) {
		d = m.getPreferredSize();
		m.setSize(d.width, d.height);
		m.setLocation(x,y);
		y+=d.height+vgap1;
	    } else {
	      continue;   // ignore other kind of Components
	    }

	    // If its a NodeContainer the line should not go to
            // the half of the NodeContainer but to the
            // half of the description of the NodeConatainer
	    xtmp=x;
	    ytmp=y;

	    if (m instanceof NodeContainer) {
	      ytmp=y-d.height-vgap1;// back to the left top corner of the node
	      Component comp;
	      anode=(NodeContainer)m;
	      comp=anode.getDescription();

	      // checkboxex are layed out from the "father" container
	      check=anode.getCheckComponent(); // important !!!!!

	      dcheck=check.getPreferredSize();
	      d=comp.getPreferredSize();
	      ytmp+=d.height/2; // add half of description height of node
	      EndPoint=new Point(xtmp-hgap2,ytmp);
	      // now we would be ready, but we have also layout
          // the CheckBox Component.
	      // I center the Checkbox to the calculated EndPoint.
	      // This means that the EndPoint changes as well, because
          // we want NOT that the line is drawn over the checkbox!
	      if (check.isVisible()) {
    		EndPoint.y-=dcheck.height/2;
    		check.setSize(dcheck.width,dcheck.height);
    		check.setLocation(EndPoint.x-dcheck.width/2,EndPoint.y);
	      }
	      // Ok now we have the right Position for the EndPoint
	      ConnPoints.addElement(StartPoint,EndPoint);
	      // ok now we draw a line to the right
	      // if the checkBox is Visible we must calculate
          // the new StartPoint right to the checkbox
          // (and in the middle of it)
	      if (check.isVisible()) {
    		StartPoint=new Point(EndPoint.x+dcheck.width/2,
				     EndPoint.y+dcheck.height/2);
	      } else {
	    	StartPoint=EndPoint;
	      }
	      EndPoint=new Point(xtmp,ytmp);
	      ConnPoints.addElement(StartPoint,EndPoint);
	      // ok now we must draw a line to the bottom
          // if we are NOT the last "stdNode" item in the
          // container
	      // First we must calculate a new StartPoint if
          // the CheckBox is visible
	      if (check.isVisible()) {
    		StartPoint=new Point(StartPoint.x-dcheck.width/2,
	    			     StartPoint.y+dcheck.height/2);
	      }
	      // Otherwise the Startpoint is set correctly!

	      for (j=i+1; j<nmembers;j++) {
    		m=node.getComponent(j);
	    	if (m.isVisible() && node.getIdentifier(m).equals("stdNode")) {
    		  // so we draw a line
    		  EndPoint=new Point(xtmp-hgap2,y);
    		  ConnPoints.addElement(StartPoint,EndPoint);
    		  break;
	    	}
	      }
	      StartPoint=EndPoint;

	    } else {
	      // an ordinary Component, we draw line to the half of it
	      ytmp=ytmp-d.height/2-vgap1;
	      EndPoint=new Point(xtmp-hgap2,ytmp);
	      // now the line from top to down
	      ConnPoints.addElement(StartPoint,EndPoint);
	      // now we have to draw to the right
	      StartPoint=EndPoint;
	      EndPoint=new Point(xtmp,ytmp);
	      ConnPoints.addElement(StartPoint,EndPoint);
	      StartPoint=new Point(xtmp-hgap2,ytmp);
	    }


	}

    }

    /**
     * Returns the String representation of this FileNodeLayout's values.
     */
    public String toString() {

	return getClass().getName() + "[hgap1=" + hgap1 + ",hgap2="+ hgap2 +",vgap1=" + vgap1 + "]";
    }
}
