/*
   NodeContainer stores Components which should be displayed in a tree
   Copyright (c) 1997 Fraunhofer-Einrichtung fuer Experimentelles
                      Software Engineering (IESE)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation.

   For more information look at the file COPYRIGHT in this package

*/



package de.unikl.awtnode;


import java.awt.*;
import java.util.*;
import java.awt.event.*;
import de.unikl.awtnode.ConnectionLinePoints;
import de.unikl.awtnode.LinePoints;
import de.unikl.awtnode.cfContainer;
import de.unikl.awtnode.pmCheckbox;




/**
 * The class NodeContainer implements a tree.
 *   <p>
 *
 *
 *   <em>What is a tree?</em><p>
 *
 *   A tree consists of nodes and leafs and edges, edges go from a node
 *   to another node (or leaf)
 *   A tree has a recusive definition:
 *   <pre>
 *          i)  A leaf is a tree
 *          ii) or it is a inner node which sons a "part"-trees
 *
 *   </pre>
 *
 *
 *   If you put a treeimplemenation into Java-class hierarchy
 *   you have to consider a few things.
 *
 *   <p>
 *   The main obstacle is that a Container in Java does not have
 *   a graphical representation of its own.
 *   A container is a component (Container extends component) but all
 *   show-methods (or in JDK1.1: setVisible(..)) only shows the
 *   components which are contained in a container.
 *   But if you look at a typical tree-implemenatation, like
 *   a file/directory tree each directory (Container) has a
 *   graphical representation of its own.
 *   Thus a directory has a graphical Component as representation.
 *
 *   <pre>
 *   eg:
 *
 *
 *   root
 *     usr
 *     var
 *     tmp
 *     bin
 *   </pre>
 *
 *   In this example the directory [root] contains the sub-directories
 *   <pre>
 *         usr,var,tmp,bin
 *   </pre>
 *
 *   If you put this together with the Java-class hierarchy, you
 *   need a show-method which:
 *   <pre>
 *       1. first displays the [root] component
 *       2. then displays the sub-container : usr,var,tmp,bin
 *   </pre>
 *
 *   It is important to understand what this mean, because
 *   now a few other problems comes up:
 *   <p>
 *
 *   <pre>
 *    1. What should be done if you call the method setVisible(false)
 *       (2 possibilities)
 *       a) only the contained components are hided (usr,var,tmp,bin)
 *       b) the contained component as well as the own graphical            
 *          representation are hided
 *
 *   </pre>
 *   The NodeContainer implements behaviour b.
 *
 *   <p>
 *   But you also need a method to realize solution a.
 *   If you look at typical tree-implementation (Windows95-explorer)
 *   there you see that in front of each directory is a box which
 *   shows a + if the directory is "closed" or a - if its "open".
 *   The Nodecontainer does the same approach. It displays in front
 *   of each node in the tree a CheckComponent. This CheckComponent
 *   does the same as the box in the W95-expoloer. This CheckComponent
 *   allows a user to select whether only the sons are hided or
 *   show and a call to setVisiblie(false) would hide the whole
 *   container as you expect it.(in the example root+[rest] are hided)
 *
 *   <p>
 *   Up to now all the main aspects are mentioned.The class
 *   ConnectionLinePoints is a rather boring class, which only
 *   stores lines. A line consists out of two points its
 *   startpoint and its endpoint. This class is used by the
 *   LayoutManager to store the connectionline between the nodes in
 *   a tree. The connectionline is painted in the NodeContainers
 *   paint method. (The connectionlines are the edges in a tree)
 *   Thus the connectionline object if filled in the LayoutManager
 *   but painted in the NodeContainer.
 *   <p>
 *   <H4>The internal implementation</H4>
 *   Like in Borderlayout I have used
 *   "identifiers" to distinguish between
 *   the "kind" of component in the container(NodeConatainer).
 *   There are three important identifiers:
 *   <pre>
 *     stdNode
 *     description
 *     check
 *   </pre>
 *   The identifer "stdNode" describes standard Components. These are
 *   the sons in a tree. A component which has the identifer "stdNode"
 *   may be a ordinary Component or a NodeContainer. All Components
 *   (sons) are added with the method :
 *   <pre>
 *   add(Component son)
 *   </pre>
 *   This call as a default the add method:
 *   <pre>
 *   add("stdNode",comp)
 *   </pre>
 *
 *   Thus each component gets its identifer "stdNode" as default.
 *   <p>
 *   But there are other identifiers like "check" and "description".
 *   If a Component has the identifer "check" this means that
 *   this component is the CheckComponnent. If you click on
 *   a CheckComponent all the sons are hided/shown. Then there
 *   is the identifier "description" this is a component which
 *   is a grpahical representation for the NodeContainer itsself.
 *   In the example the components: root,usr, var... would have
 *   the following identifiers:
 *   <pre>
 *   ("description",root)
 *   ("stdNode",usr)
 *   ("stdNode",var)
 *   (But there are other elements in the container, described
 *   later in the text)
 *   </pre>
 *
 *   In general you have not to deal with the identifiers "check"
 *   and "description".
 *
 *
 *   <p>
 *   Now a few words how a tree is actuallised.In genral you can say
 *   that the whole tree has to be layouted if you add another
 *   Component to the tree or if you delete (remove) a component.
 *   Thus we need a mechanism which lays out all the Node from
 *   the root Node of the tree to the leafs. This is done
 *   with the method :
 *   <pre>
 *   getRoot()
 *   </pre>
 *
 *   With this method you get the root node of the whole tree, even
 *   if you invoke this call on a leaf in your tree.(internally
 *   the method walk up the NodeContainer hierarchy until there
 *   is no NodeContainer as a parent)
 *
 *   <p>
 *   Now a word about the CheckComponents:
 *   It would be nice if the root-Node, which has no NodeContainer
 *   as a parent may have a CheckComponent in front of it. In W95
 *   exporer this is possible. If the user click on this checkbox
 *   the whole tree is hided. To implement this there is another
 *   identifier. Apart from the idntifers "stdNode","description"
 *   and "check" there is a new identifier "thischeck".
 *   If a Component has the identifier "thischeck" this means, that
 *   this Component is the Checkbox for the "root" Node of the
 *   tree.
 *   <p>
 *   <H4>But what happens if you merge a tree with another?</H4>
 *
 *   <pre>
 *   Answer:
 *   The Checkbox is transferred between the NodeContainers.
 *   </pre>
 *   <pre>
 *   Example:
 *   </pre>
 *   You have a NodeContainer B and a NodeContainer A. And you call:
 *   <pre><code>
 *   A.add(B);
 *   </pre></code>
 *   Then the "thischeck"-box from NodeContainer B is deleted
 *   from the NodeContainer and is inserted in NodeContainer A
 *   but with the Identifier "check"
 *   Now you call :
 *   <pre><code>
 *   A.remove(B);
 *   </pre></code>
 *   then the Checkbox for B (stored in A) is removed from A
 *   and is insert with the indentifer "thischeck" in B.
 *   Thus you can say: If you build a tree the checkboxe are
 *   transferred between the NodeContainers. (This approach
 *   makes the Layou-process more easily)
 *
 **/









public abstract class NodeContainer extends cfContainer
                                    implements ItemListener {

  /**
   *  the graphical representation of this Container
   */
  Component description;


  /**
   * the CheckComponent only hides the added-Components
   */
  pmCheckbox CheckComp;


  /**
   * this stores the Lines between the Nodes in a tree.
   * Filled by LayoutManger. The setVisible(true)-Method of the
   * NodeContainer draws these lines
   */

  ConnectionLinePoints connPoints;





  /**
    Constructs a NodeContainer with a description-Component.
    The default Layout is : FileTreeLayout
    The default value for the Checkbox is : true

    */

  public NodeContainer(Component description) {

    this.description=description;

    // we create a Checkbox, and add it with the identifer "thischeck"
    // If we add this NodeContainer to another NodeContainer
    // the add-mechanism gets this Checkbox and
    // adds it to the new NodeContainer, but with the identifier "check"
    // then this Checkbox in this container is removed
    CheckComp=new pmCheckbox();
    super.add("thischeck",CheckComp);
    CheckComp.setState(true);       // initialize in "close" state
    CheckComp.addItemListener(this);

    connPoints = new ConnectionLinePoints();
    connPoints.setParent(this);
    setLayout(new FileNodeLayout());
    //setLayout(new FlowLayout());

    super.add("description",this.description);

  }



  /**
   * Adds Component to the tree.
   *
   * Default specifier is "stdNode".
   */

 /*
   * Note:
   *
   * ALL ADD METHODS ARE FINAL!
   *
   * final, because we implement a tree, and insertion operations
   * on a tree have the name "insert" and not "add"
   * In derived classes we only use insert. There is NO need to
   * overload "add" Methods
   *
   * The added Component is hided() or shown() so it fits in the
   * NodeContainer. If a Component is hided() but the CheckComp
   * of the Node is "show" the Component gets schown as well
   */


  public final Component add(Component comp) {
    Component back;

    if (comp instanceof NodeContainer) {
    	registerNodeContainer((NodeContainer)comp,-1); // insert at end
    	resetComponent(comp);
    	return comp;
    }
    back=super.add("stdNode",comp);
    resetComponent(comp);
    return back;
  }


  /**
   * Adds Component under the specified classification.
   * Note:
   * it ist possible to pass a NodeContainer in the Constructor
   * of a NodeContainer.(as a description)
   *
   * @param name the classification ("stdNode","description","check")
   * @param comp the Component to add
   */
  public final Component add(String name,Component comp) {
    Component back;

    if ((comp instanceof NodeContainer) &&
    	(name.equals("stdNode"))) {
      // perhaps we add NodeContainer as a description,
      // then we would add it NOT as a "stdNode"
      // (erverything is possible)
      registerNodeContainer((NodeContainer)comp,-1); // insert at end
      resetComponent(comp);
      return comp;
    }
    if (name.equals("description")) {
      System.err.println("descriptions can only be set in the Constructor OR");
      System.err.println("in method setDescription!\n\n");
      System.out.println("current Node: "+toString());
      System.err.println("added Component: "+comp.toString());
      System.err.println("add is ***** ignored *****");
    }
    back=super.add(name,comp);
    resetComponent(comp);
    return back;
  }

  /**
   * Adds Component as "stdNode" to the NodeConatainer
   */


  public final Component add(Component comp, int pos) {
    Component back;

    if (comp instanceof NodeContainer) {
	    registerNodeContainer((NodeContainer)comp,pos); // insert at end
    	resetComponent(comp);
	return comp;
    }
    back=super.add(comp,pos);
    resetComponent(comp);
    return back;
  }

  /**
   * if you add a new Component to a NodeContainer with a "closed"
   * checkbox the added componetn must be hided().
   * Or if you add a hided() componetn to a NodeContainer
   * with a "open" checkbox then we show the component.
   * This method cares about this. We adjust the component
   * to the NodeContainer's needs
   */
  private void resetComponent(Component newComp) {
    NodeContainer node;
    boolean lvis=CheckComp.isVisible();
    boolean lstate=CheckComp.getState();


    if (newComp == description) return;

      // if Checkbox not vis => SubComp not visible
      // if Checbox vis and Checkbox status == true =>SubComp not visible
      if ((lvis == false) || (lstate == true)){
         newComp.setVisible(false);
         if (newComp instanceof NodeContainer) {
            ((NodeContainer)newComp).getCheckComponent().setVisible(false);
         }
      } else {
	  // else Checkbox visible and "open"
	  newComp.setVisible(true);
	  if (newComp instanceof NodeContainer) {
	      ((NodeContainer)newComp).getCheckComponent().setVisible(true);
	  }
    }

  }




  /**
   * This is tricky. If we add a NodeContainer to another Nodecontainer
   * we have a problem. The kind of Layout I have implemented
   * forces me to lay out the Checkbox in the parent container.
   * But if the Checkbox belongs to the son, all coordinates
   * will transferred into his coordinate space.(The Checkbox
   * is drawn at the wrong place)
   * Thus the Checkbox has to be transferred to this Nodecontainer
   * and must be removed from the "son" NodeConatiner.
   * I get the Pointer of the Checkbox in the son, and add
   * this Pointer to this Nodecontainer (with the identifier "check")
   * As a result the Checkbox
   * is drawn in this coordinate space. (I haven't yet found out
   * how to create a Component peer which doesn't belong to a Container,
   * if this is possible I can draw the Checkbox independed of the
   * actual NodeContainer. Hints are welcome)
   *
   * @param pos the position where the Node is added
   * @return the node argument
   */
  private void registerNodeContainer(NodeContainer node, int pos) {
    pmCheckbox soncheck;
    soncheck=node.getCheckComponent();
    // we only can add a NodeContainer if the Pointer of it
    // is not already contained. This must be checked, because
    // you can add a NodeContainer as a description!
    if (isContained(node) == true) {
      return;
    }


    // add the Checkbox of the son to this Node under ident. "check"
    // and remove it from the son
    super.add("check",soncheck); // added at last Position
    node.remove(soncheck);


    // if node already added, we won't add it another time !!!
    // this is checked in the add-method, via the identifier Hashtable
    super.add(node,pos);  // perphaps we must add 1 because soncheck counts +1
    setIdentifier("stdNode",node);

  }





  /**
   * We remove first the Component, and if the Component is
   * an instance of NodeContainer, we must also remove
   * the Checkbox and add it to the son under the identifier "thischeck"
   * We add it at the first position!
   */

  /*
   * remove methods are final. The NodeContainer should appear
   * as Component, not as a container!
   */

  public final void remove (Component comp) {
    NodeContainer node;
    pmCheckbox check;

    if (comp instanceof NodeContainer) {
      node=(NodeContainer)comp;
      check=node.getCheckComponent();
      super.remove(check);
      node.add(check,0); // add at first Position (with ident. "stNode")
      node.setIdentifier("thischeck",check);
    }

    super.remove(comp);
  }



  /**
   * remove all Components from this NodeContainer.
   *
   * Note:
   * The Checkboxes are transferred to their "real" owners.
   * (this is because we have to overload the method)
   */
  public final void removeAll() {
    int i;
    int n=getComponentCount();
    for(i=0;i<n;i++) {
      Component m=getComponent(i);
      remove(m);
    }
  }


  /**
   * select whether the root-Node draws a Checkbox or not
   *
   * If the NodeContainer has a parent, which is no NodeContainer,
   * then this Nodecontainer is the root-Node of all its "sons"
   * Sometime yout want the Checkbox to this NodeContainer in
   * order to setVisible(false) the whole tree. You can select wether this
   * checkbox is drawn with this NodeContainer(corressponding to the
   * Layoutmanager, or not. The default is : true, we draw it
   *
   * @param lpaint true, paints the Checkbox in front of the root-Node
   */
  public final void setRootCheck(boolean lpaint) {
    CheckComp.setPaint(lpaint);
  }


  /**
   * returns whether we draw a Checkbox in front of the root-Node or not
   */
  public final boolean getRootCheck() {
    return CheckComp.getPaint();
  }



  /**
   * usually only used by the <XYZ>NodeLayout Managers, when
   * the layout the "ConnectionLine" between the Nodes of the tree
   * @returns ConnectionLinePoints-object
   */

  public final  ConnectionLinePoints getConnectionLinePoints() {
    return connPoints;
  }



  /**
   * returns the description Component
   */
  public final Component getDescription() {
    return description;
  }


  /**
   * sets a new Description Component for this NodeContainer
   */
  public final void setDescription(Component newDesc) {
    remove(description);
    description=newDesc;

    // Now the insertion .....
    // attention: we cannot call add(newDesc,"description");
    // because the the "user" can call this to, without
    // knowing what he does. (we remind him to call this
    // method if he tries to call add(..))


    // [insert-operation -start]

    super.add("description",newDesc);

    // [insert-operation end]

    invalidate();
  }


  /**
   * returns the correspondig NodeContainer for the given Checkbox.
   *
   * the getParent() method of a checkbox does NOT return
   * the NodeContainer which it belongs to, but
   * the parent NodeContainer! With this method you get the
   * "real" NodeContainer.
   *
   * @param Check the Checkbox for which we determine the NodeContainer
   * @returns the correspoding NodeContainer  we found or null otherwise
   */


  /*
   * A Checkbox AND the NodeContainer to which it belong
   * are both Components in the "Father" NodeContainer.("check","stdNode")
   * Thus in a handleEvent-Checkbox-klick, the father Node
   * gets the Event, "real" Node. But we pass it to the "real" Node
   *
   * see: handleEvent
   *
   */
  public NodeContainer getSonNodeContainer(pmCheckbox check) {
    int i;
    int n=getComponentCount();
    Component m;
    NodeContainer node;
    pmCheckbox aCheckBox;

    // perhaps we do a speedup, if we store this
    // relation (check,sonNode) directly....
    for(i=0;i<n;i++) {
      m=getComponent(i);
      if (m instanceof NodeContainer) {
    	node=(NodeContainer)m;
    	aCheckBox=node.getCheckComponent();
	    if (aCheckBox == check) {
    	  return node;
    	}
      }
    }
    return null;
  }




  /**
   * returns the Check Component
   */
  public pmCheckbox getCheckComponent() {
    return CheckComp;
  }



  /**
   * if set to true, alle Components IN the Node get setVisible(false)
   */
  public void setState(boolean check) {
    NodeContainer root;
    Dimension dim;

    CheckComp.setState(check);
    if (check == true) {
      hideSons();
    } else {
      showSons();
    }
    invalidate();

    // now we take care that we get resized by the parent
    // of the root-NodeConatiner
    System.out.println("hello1");
   	root=getRoot();
   	System.out.println("hello2");
   	dim=root.getPreferredSize();
   	root.setSize(dim);
   	System.out.println("hello3");
   	root.getParent().validate();
   	System.out.println("hello4");

  }



  /**
   * @returns the state of the Checkbox(!)
   */

  public boolean getState() {
    return CheckComp.getState();
  }



    /*
     * The NodeContainers build a tree. If you have a NodeContainer
     * it is not clear whether it is a "inner Node" of the tree
     * or a "leaf" or the "root". With this method you
     * can determine the "root" Node of this Node.
     * The root Node of a tree is (in this context) the
     * "last" NodeContainer which has no NodeContainer as a
     * parent.
     * @returns the root NodeContainer of this NodeContainer
    */
  public NodeContainer getRoot() {
    NodeContainer node;
    Component tmp;

    tmp=getParent();
    if ((tmp instanceof NodeContainer) == false) {
      return this;
    }
    node=(NodeContainer)tmp;
    return node.getRoot();
  }




  public void showSons() {
    int i;
    int n;
    Component m;

    if (CheckComp.getState()==false) {
      n=getComponentCount();
      connPoints.setVisible(true);
      for(i=0;i<n;i++) {
    	m=getComponent(i);
    	if ( (m != CheckComp) && m != description) { // is m a son?
    	  m.setVisible(true);
    	  m.invalidate();
    	}
      }
    }

  }

  public void hideSons() {
    int i;
    int n;
    Component m;

    if (CheckComp.getState()==true) {
      n=getComponentCount();
      connPoints.setVisible(false);
      for(i=0;i<n;i++) {
    	m=getComponent(i);
    	if ( (m != CheckComp) && m != description) { // is m a son?
    	  m.setVisible(false);
    	}
      }
    }

  }

    /**
     * Validates this Container and all of the components contained
     * within it.
     * First all sons are validated, the the NodeContainer is
     * validated.
     * @see #validate
     * @see Component#invalidate
     */
    public void validate() {
        int i;
        int n=getComponentCount();
        for (i=0;i<n;i++) {
            Component m=getComponent(i);
            m.validate();
        }
        super.validate();
    }


    /**
     * PAINTS THE CONNECTION-LINE.The ConnectioLine is stored as a Structure
     * of Points. The x,y Points gets calculated by the
     * LayoutManager, then it must be drawn in this paint-Method.
     *
     * @param g the specified Graphics window
     * @see Component#paint
     * @see Component#paintAll
     */


  /*
     * Internals:
     * One Exception is made because of the line to the "root" Node.
     * In a proper implementation the programmer should select for
     * each Nodecontainer whether a linie from the
     * description component to the checkbox is painted or not.
     * In the current implementation this is not possible.
     * You can only select whether to draw a line for all
     * sons of a NodeContainer or not. (This fits for the most cases)
     * But with this a problem enconters if you hide the whole
     * connection line for a NodeConateiner if this NodeContainer
     * is the "root" NodeContainer and normally has a Connection
     * line and a "root" Checkbox:
     * The line from the "root" Checkbox to the description
     * Component would disapper when you hide the Subcomponent!
     * We fix this(quick and dirty)
     */
    public void paint(Graphics g) {

      connPoints.paint(g);

      // dirty hack [start]
      if (CheckComp.getParent() == this) { // check belongs to this NodeCon?
	  if ((CheckComp.getPaint() == true) && (CheckComp.isVisible())){
	    LinePoints line;
	    ConnectionLinePoints cline=new ConnectionLinePoints();

	    Rectangle rec1=CheckComp.getBounds();
	    Rectangle rec2=description.getBounds();
	    int x1=rec1.x+rec1.width;
	    int y1=rec1.y+rec1.height/2;
	    int x2=rec2.x;
	    int y2=y1;

	    line=new LinePoints(new Point(x1,y1),new Point(x2,y2));

	    cline.addElement(line.a,line.b);
	    cline.setParent(this);
	    cline.paint(g);
	  }
      }
      // dirty hack [end]
      super.paint(g);
    }






  // I think its a good idea to put the handleEvent stuff
  // at the end

  /**
   * we implement the Standard behavior if a checkBox is clicked
   */
  public void itemStateChanged(ItemEvent e) {
    if (e.getStateChange() == ItemEvent.SELECTED) {
        setState(true);
    } else {
        setState(false);
    }
  }


    /**
     * Returns the String representation of this NodeContainer values.
     */
    public String toString() {
        return getClass().getName() + "[" + description.toString()+","
	                                  + CheckComp.toString()+","
	                                  + super.toString() + "]";
    }



}


