/*
   RescanNode adds re-load capapilities to a NodeContainer
   Copyright (c) 1997 Fraunhofer-Einrichtung fuer Experimentelles
                      Software Engineering (IESE)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation.

   For more information look at the file COPYRIGHT in this package

*/


package de.unikl.awtnode;


import java.awt.*;
import java.util.*;
import de.unikl.awtnode.NodeContainer;



/**
 * This class implements the possibility that a node in a
 * tree may create it sons, if you "open" the node.
 * <p>
 * Assume you have a directory tree view. If you look at the
 * files it is clear that they can be deleted or renamed but
 * if they are stored in your tree and you delete the correspondig file
 * on the harddisk the tree is not updated!
 * Thus there must be a mechanism which can be used do update
 * a tree, or better to re-create a tree.
 *
 * <p>
 * To achieve this goal you can use the new Variable [lRescan]:
 * <p>
 * Each time you "open" the NodeContainer (this means
 * the state of the checkbox changes from true to false)
 * the Object checks whether the variable lRescan is true
 * or not. If lRescan is true, this means that
 * the node should be updated and theObject calls the method Rescan.
 * <p>
 *
 * the default in the method Rescan is : do nothing
 * As an example a fairly simple implementation for a Directory
 * would do the following steps:<p>
 * <pre><code>
 * simpliest solution
 * 1. remove all sons of this node
 * 2. re-read the directory
 * 3. insert all the sons in the directoy to the current node
 * </pre></code>
 * (this solution is simple and slow)

 */



public abstract class RescanNode extends NodeContainer {


  /**
   * if true, we call <Rescan> , when the NodeContainer is opened
   */


  boolean lRescan;


  public RescanNode(Component desc) {
    super(desc);
  }



    /**
    * if change to true, and lRescan == true we do a Rescan on the Node
    */
  public void setState(boolean check) {
    boolean state=getState();   // actual state we are currently in
    if ( (state == true) && (check == false)) { // change to "open"
      if (lRescan == true) {
    	Rescan();
      }
    }
    super.setState(check);
  }


  /**
   * Overload this, if you want "rescan" abilities
   */

  public void Rescan() {
    // no action
  }

  /**
   * indicates wheter we call <Rescan> when a NodeContainer is "opened"
   **/

  public boolean getRescan() {
    return lRescan;
  }


  /**
   * Sets the Rescan Variable for this NodeContainer
   */

  public void setRescan(boolean lRescan) {
    this.lRescan=lRescan;
  }



}
