/*
   a plus/minus Component (similar to a CheckComponent)
   Copyright (c) 1997 Fraunhofer-Einrichtung fuer Experimentelles
                      Software Engineering (IESE)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation.

   For more information look at the file COPYRIGHT in this package

*/

package de.unikl.awtnode;


import java.awt.*;
import java.awt.event.*;

/**
 *	pmCheckbox  = plus/minus Checkbox
 *  <p>
 *	The Checkbox draws a + if its state ist true; otherwise
 *	it draws a -.
 *   <p>
 *	The switches setPaint/getPaint sets wether the Checkbox
 *	would ever draw itsself, regardless of if  its visible.
 *	You can set setPaint(false) and you have no Checkbox
 * 	on the screen, although you call the setVisible(true) method.
 *
 */


public class pmCheckbox extends Canvas
                        implements  ItemSelectable,
                                    MouseListener {


  /**
   * a checkbox has two different states, checked and not checked
   **/

  boolean state;


  /**
   * if lpaint is false we dont paint the checkbox, even
   * if setVisible(..) is true
   **/
  boolean lpaint;



  transient ItemListener itemListener;


  /**
   * Konstrukts a new Checkbox, with the default size : (9,9)
   * initial state=false
   */
  pmCheckbox() {
    super();
    setSize(9,9);
    setState(false);
    setPaint(true);
    addMouseListener(this);
  }


  /**
   * Sets the Checkbox to the specifed boolean state.
   * @param state the boolean state
   * @see #getState
   */

   public void setState(boolean state) {
     this.state=state;
     invalidate();
     repaint();
   }


    /**
     * Returns the boolean state of the Checkbox.
     * @see #setState
     */
    public boolean getState() {
        return state;
    }


  /**
   * paints the checkbox corresponding to the state: true =+, false = -
   *
   * Note:
   * if getPaint() == false, nothing is drawn at all!
   */

  public void paint(Graphics g) {
    Rectangle rec=getBounds();
    int gap;
    int x;
    int y;

    if (lpaint == false) return;

    g.drawRect(0,0,rec.width-1,rec.height-1);

    gap=rec.width/3;
    y=(rec.height/2);
    x=(rec.width/2);
    g.drawLine(gap-1,y,rec.width-gap,y);  // a -

    if (getState()==true) {
      g.drawLine(x,gap-1,x,rec.height-gap);
    }

  }




  /**
   * This sets wether the Checkbox is painted  at all.
   * true means: paint is possible
   * false measn: we never paint it, even not when you call paint()
   * @param lpaint sets wether we draw the Checkbox or not
   */

  public void setPaint(boolean lpaint) {
    this.lpaint=lpaint;
    invalidate();
  }



  /**
   *
   * @return the current Paint-Status flag
   */
  public boolean getPaint() {
    return lpaint;
  }


  // Interface:  ItemSelectable [Start]

    /**
     * Adds the specified item listener to recieve item events from
     * this checkbox.
     * @param l the item listener
     */
    public void addItemListener(ItemListener l) {
        itemListener = AWTEventMulticaster.add(itemListener, l);
    }

    /**
     * Removes the specified item listener so that it no longer
     * receives item events from this checkbox.
     * @param l the item listener
     */
    public void removeItemListener(ItemListener l) {
        itemListener = AWTEventMulticaster.remove(itemListener, l);
    }


   /**
     * Returns the an array (length 1) containing the checkbox
     * label or null if the checkbox is not selected.
     * @see ItemSelectable
     */
    public Object[] getSelectedObjects() {
        if (state) {
            Object[] items = new Object[1];
            items[0] = new String("true");
            return items;
        }
        return null;
    }

    // Interface:  ItemSelectable [End]


    // Interface : MouseListener [Start]

    /**
     * The mouse click on a pmCheckbox ist mapped
     * to an ItemEvent
     */
    public void mouseClicked(MouseEvent e) {
      ItemEvent ev;
      //System.out.println("Mouse clicked");
      if (state == true) {
        ev=new ItemEvent(this,ItemEvent.ITEM_STATE_CHANGED,"",
                            ItemEvent.DESELECTED);
      } else {
        ev=new ItemEvent(this,ItemEvent.ITEM_STATE_CHANGED,"",
                            ItemEvent.SELECTED);
      }
      processEvent(ev);
    }

    /**
     * no functionality behind
     */
    public void mousePressed(MouseEvent e) {
      //System.out.println("Mouse pressed");
    }

    /**
     * no functionality behind
     */
    public void mouseReleased(MouseEvent e) {
      //System.out.println("Mouse released");
    }


    /**
     * no functionality behind
     */
    public void mouseEntered(MouseEvent e) {
      //System.out.println("Mouse entered");
    }

    /**
     * no functionality behind
     */
    public void mouseExited(MouseEvent e) {
      //System.out.println("Mouse exited");
    }

    // Interface : MouseListenr [End]





    /**
     * Processes events on this checkbox. If the event is an ItemEvent,
     * it invokes the handleItemEvent method, else it calls its
     * superclass's processEvent.
     * @param e the event
     */
    protected void processEvent(AWTEvent e) {
        if (e instanceof ItemEvent) {
            processItemEvent((ItemEvent)e);
            return;
        }
        super.processEvent(e);
    }

    /**
     * Processes item events occurring on this checkbox by
     * dispatching them to any registered ItemListener objects.
     * NOTE: This method will not be called unless item events
     * are enabled for this component; this happens when one of the
     * following occurs:
     * a) An ItemListener object is registered via addItemListener()
     * b) Item events are enabled via enableEvents()
     * @see Component#enableEvents
     * @param e the item event
     */
    protected void processItemEvent(ItemEvent e) {
        if (itemListener != null) {
            itemListener.itemStateChanged(e);
        }
    }



    public String toString() {
        String str = super.paramString();
        return str + ",state=" + state;
    }


}







